module Main ( main
            ) where

import Algos.DivideAndConquer.Sorting (mergeSort, merge)
import Algos.Graphs.Forest (Tree(..), Subtree(..))
import Algos.Graphs.Graph (Graph(..), Vertex(..), Edge(..))
import Algos.Graphs.GraphIO (Display(..), Orientation(..), displayGraph)
import Algos.Random.Numbers (randomNumberM)
import Algos.Random.Partitions (randomPartitionM, splitBySize, randomSegmentedSampleM, randomSegmentedSumM)
import Algos.Random.Sequences (randomPermutationM)
import Control.Applicative ((<$>), (<*>))
import Control.Exception (handle, SomeException(..))
import Control.Monad.State (State, evalState)
import Data.Array (listArray, (!))
import Data.List (intercalate, unfoldr)
import Data.Map (fromAscList, lookupLT)
import System.Environment (getArgs, getProgName)
import System.Exit (exitFailure)
import System.IO (withFile, IOMode(..), hPutStr)
import System.Random (newStdGen, RandomGen)

main :: IO ()
main =  do (n, m, c, gfile, ccfile) <- readArgs
           gen                      <- newStdGen
           let (ccs, g) = flip evalState gen $ do ccs <- genCCs n m c
                                                  g   <- joinCCs ccs
                                                  return (ccs, g)
           withFile gfile  WriteMode $ 
             \h -> hPutStr h (displayGraph (Undirected, g))
           withFile ccfile WriteMode $ 
             \h -> hPutStr h ( intercalate "\n" $ 
                               map (\cc -> displayGraph (Undirected, cc)) ccs
                             )

readArgs :: IO (Int, Int, Int, String, String)
readArgs =  handle (\(SomeException _) -> usage >> exitFailure) $
            do [nstr, mstr, cstr, gfile, ccfile] <- getArgs
               let n = read nstr
                   m = read mstr
                   c = read cstr
               return (n, m, c, gfile, ccfile)

usage :: IO ()
usage =  do prog <- getProgName
            putStrLn $ "USAGE: " ++ prog ++ " <n> <m> <c> <graph file> <cc file>"

genCCs       :: RandomGen g => Int -> Int -> Int -> State g [Graph Int Int Int]
genCCs n m c =  do vss <- randomPartitionM c [1..n]
                   let ns     = map length vss
                       minms  = map (subtract 1) ns
                       maxms  = map (\n -> n * (n-1) `div` 2) ns
                       freems = zipWith (-) maxms minms
                       minm   = sum minms
                   ms  <- zipWith (+) minms <$> randomSegmentedSumM (m - minm) freems
                   ess <- splitBySize ms <$> randomPermutationM [1..sum ms]
                   sequence (zipWith genCC vss ess)
                       
genCC            :: RandomGen g => [Int] -> [Int] -> State g (Graph Int Int Int)
genCC vids elbls =  do let n = length vids
                           m = length elbls
                       ps   <- mapM (\v -> randomNumberM (1, v-1)) [2..n]
                       npss <- zipWith adjustForParent (tail ps) <$> 
                               randomSegmentedSampleM (m - n + 1) [1..n-2]
                       let nbs = zipWith (:) ps ([] : npss)
                           eps = concat $ zipWith (\us v -> zip us (repeat v)) nbs [2..n]
                           va  = listArray (1, n) vids
                           vs  = map (\i -> Vertex i 0) vids
                       es <- sequence $ zipWith (mkEdge va) eps elbls
                       Graph vs <$> randomPermutationM es
                      
  where adjustForParent p nps = map adj nps
          where adj np | np < p    = np
                       | otherwise = np + 1

        mkEdge va (u, v) lbl = do f <- randomNumberM (0,1)
                                  case f of
                                    0 -> return (Edge (va!u) (va!v) lbl)
                                    1 -> return (Edge (va!v) (va!u) lbl)

joinCCs     :: RandomGen g => [Graph Int Int Int] -> State g (Graph Int Int Int)
joinCCs ccs =  Graph <$> randomPermutationM (concatMap graphVertices ccs)
                     <*> randomPermutationM (concatMap graphEdges    ccs)

instance Display Int where
  display = show
