{-------------------------------------------------------------------------------

A wrapper for the connected components algorithm so it can turn an input graph
read from file into a list of connected components.  The input graph is read
from file, which must have the following structure:

- One line storing the word "Directed" or "Undirected" and two integers, one the
  number n of vertices, the other the number m of edges.  (This is a consequence
  of using Algos.GraphIO.parseGraph to read the graph.  This labelling is ignored
  by the algorithm.)

- n lines, one per vertex, storing the ID and label of each vertex separated by
  spaces.

- m lines, one per edge, storing the ID of the tail, the ID of the head, and the
  label separated by spaces.

This code makes no effort to check that the input is actually valid.  For
example, if fewer than m edge lines are created, the code will simply raise an
exception and terminate abnormally.

The output components are represented as pairs of vertex and edge lists.

-------------------------------------------------------------------------------}

{-# LANGUAGE FlexibleInstances #-}

module Main ( main
            ) where

import Algos.Graphs.ConnectedComponents (ccPartition)
import Algos.Graphs.Graph (Graph, AdjList(..), V, E, makeUndirAdjListIx, makeUndirGraph)
import Algos.Graphs.GraphIO (Display(..), Parse(..), parseGraph, displayGraph, Orientation(..))
import Control.Applicative ((<$>))
import Data.List (intercalate)
import System.Environment (getArgs, getProgName)
import System.Exit (exitFailure)
import System.IO (withFile, IOMode(..), hPutStr)

-- The main function
main :: IO ()
main =  do (gfile, ccfile) <- readArgs
           g <- parseGraph <$> readFile gfile 
           writeCCs ccfile (runCC g)

readArgs :: IO (String, String)
readArgs =  do args <- getArgs
               case args of
                 [gfile, ccfile] -> return (gfile, ccfile)
                 _               -> usage >> exitFailure

-- Prints a usage message in case the command line arguments make no sense
usage :: IO ()
usage =  do prog <- getProgName
            putStrLn $ "USAGE: " ++ prog ++ " <input file> <output file>"
           
-- Prints a given graph to stdout
writeCCs           :: String -> [Graph Int Int Int] -> IO ()
writeCCs fname ccs =  withFile fname WriteMode $
                      \h -> hPutStr h (intercalate "\n" $ map (curry displayGraph Undirected) ccs)

-- Computes the connected components
runCC        :: (Orientation, Graph Int Int Int) -> [Graph Int Int Int]
runCC (_, g) =  map makeUndirGraph as
  where a  = makeUndirAdjListIx g
        as = ccPartition a

-- A parse instance for strings that doesn't require quotation marks
instance Parse Int where
  parse = read

-- A display instance for strings that doesn't print quotation marks
instance Display Int where
  display = show
