{-# LANGUAGE StandaloneDeriving, FlexibleInstances #-}

module Main ( main
            ) where

import Algos.DivideAndConquer.Sorting (mergeSort, mergeSortBy)
import Algos.Graphs.Graph (Graph(..), Vertex(..), Edge(..))
import Algos.Graphs.GraphIO (parseGraph, displayGraph, Orientation(..), Display(..), Parse(..))
import Control.Applicative ((<$>))
import qualified Data.ByteString.Char8 as BS
import Data.Function (on)
import Data.List (find, unfoldr)
import System.Environment (getArgs, getProgName)
import System.Exit (exitFailure)

main :: IO ()
main =  do (outfile, expfile) <- readArgs
           ccs    <- parseCCs <$> BS.readFile outfile
           expccs <- parseCCs <$> BS.readFile expfile
           maybe (putStrLn "OK") printMismatch (verifyCCs ccs expccs)

readArgs :: IO (String, String)
readArgs =  do args <- getArgs
               case args of
                 [outfile, expfile] -> return (outfile, expfile)
                 _                  -> usage >> exitFailure

usage :: IO ()
usage =  do prog <- getProgName
            putStrLn $ "USAGE: " ++ prog ++ " <output file> <expected output file>"

parseCCs :: BS.ByteString -> [Graph Int Int Int]
parseCCs =  map (snd . parseGraph . BS.unpack) . unfoldr cc . dropWhile BS.null . BS.lines
  where cc [] = Nothing
        cc ls = Just (BS.unlines hd, dropWhile BS.null tl)
          where (hd, tl) = break BS.null ls

printMismatch          :: (Graph Int Int Int, Graph Int Int Int) -> IO ()
printMismatch (g1, g2) =  do putStrLn "MISMATCH OF CONNECTED COMPONENTS:\n"
                             putStrLn "FOUND:"
                             putStrLn (displayGraph (Undirected, g1))
                             putStrLn "EXPECTED:"
                             putStr   (displayGraph (Undirected, g2))

verifyCCs            :: [Graph Int Int Int] -> [Graph Int Int Int] -> 
                        Maybe (Graph Int Int Int, Graph Int Int Int)
verifyCCs ccs expccs =  find notSame (zip sccs sexpccs)
  where nccs                = map normalizeGraph ccs
        nexpccs             = map normalizeGraph expccs
        sccs                = mergeSortBy (compare `on` vertexId . head . graphVertices) nccs
        sexpccs             = mergeSortBy (compare `on` vertexId . head . graphVertices) nexpccs
        notSame (cc, expcc) = graphVertices cc /= graphVertices expcc ||
                              graphEdges    cc /= graphEdges expcc

normalizeGraph :: Graph Int Int Int -> Graph Int Int Int
normalizeGraph (Graph vs es) =  Graph (mergeSort vs) (mergeSort es')
  where es' = map normalizeEdge es
        normalizeEdge (Edge t h l) = Edge (min t h) (max t h) l

instance Display Int where
  display = show

instance Parse Int where
  parse = read

deriving instance Eq (Vertex Int Int)
deriving instance Eq (Edge Int Int)

deriving instance Ord (Vertex Int Int)
deriving instance Ord (Edge Int Int)